@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package Test::Deep::JType;
 # ABSTRACT: Test::Deep helpers for JSON::Typist data
-$Test::Deep::JType::VERSION = '0.003';
+$Test::Deep::JType::VERSION = '0.004';
 use Test::Deep 1.123 (); # for LeafWrapper and obj~~re diagnostics
 
 use Exporter 'import';
@@ -86,6 +86,12 @@ sub jcmp_deeply {
 #pod C<jtrue> and C<jfalse> are shorthand for C<jbool(1)> and C<jbool(0)>,
 #pod respectively.
 #pod
+#pod As long as they've got a specific value to test for (that is, you called
+#pod C<jstr("foo")> and not C<jstr()>, the tests produced by these routines will
+#pod serialize via a C<convert_blessed>-enabled JSON encode into the appropriate
+#pod types.  This makes it convenient to use these routines for building JSON as
+#pod well as testing it.
+#pod
 #pod =cut
 
 my $STRING = Test::Deep::obj_isa('JSON::Typist::String');
@@ -95,26 +101,9 @@ my $BOOL   = Test::Deep::any(
   Test::Deep::obj_isa('JSON::PP::Boolean'),
 );
 
-sub jstr  {
-  return( @_
-          ? Test::Deep::all( $STRING, Test::Deep::str(@_) )
-          : $STRING
-  );
-}
-
-sub jnum  {
-  return( @_
-          ? Test::Deep::all( $NUMBER, Test::Deep::num(@_) )
-          : $NUMBER
-  );
-}
-
-sub jbool {
-  return( @_
-          ? Test::Deep::all( $BOOL, Test::Deep::bool(@_) )
-          : $BOOL
-  );
-}
+sub jstr  { Test::Deep::JType::jstr->_new(@_);  }
+sub jnum  { Test::Deep::JType::jnum->_new(@_);  }
+sub jbool { Test::Deep::JType::jbool->_new(@_); }
 
 my $TRUE  = jbool(1);
 my $FALSE = jbool(0);
@@ -122,6 +111,69 @@ my $FALSE = jbool(0);
 sub jtrue  { $TRUE  }
 sub jfalse { $FALSE }
 
+{
+  package Test::Deep::JType::jstr;
+$Test::Deep::JType::jstr::VERSION = '0.004';
+BEGIN { our @ISA = 'Test::Deep::All'; }
+  sub TO_JSON {
+    Carp::confess("can't use valueless jstr() test as JSON data")
+      unless defined $_[0]->{JType_value};
+    return "$_[0]->{JType_value}";
+  }
+  sub _new {
+    my $class = shift;
+    my $test = Test::Deep::all(
+      $STRING,
+      (@_ ?  Test::Deep::str($_[0]) : ()),
+    );
+    $test->{JType_value} = $_[0];
+    bless $test, $class;
+    return $test;
+  }
+}
+
+{
+  package Test::Deep::JType::jnum;
+$Test::Deep::JType::jnum::VERSION = '0.004';
+BEGIN { our @ISA = 'Test::Deep::All'; }
+  sub TO_JSON {
+    Carp::confess("can't use valueless jnum() test as JSON data")
+      unless defined $_[0]->{JType_value};
+    return 0 + $_[0]->{JType_value};
+  }
+  sub _new {
+    my $class = shift;
+    my $test = Test::Deep::all(
+      $NUMBER,
+      (@_ ?  Test::Deep::num($_[0]) : ()),
+    );
+    $test->{JType_value} = $_[0];
+    bless $test, $class;
+    return $test;
+  }
+}
+
+{
+  package Test::Deep::JType::jbool;
+$Test::Deep::JType::jbool::VERSION = '0.004';
+BEGIN { our @ISA = 'Test::Deep::All'; }
+  sub TO_JSON {
+    Carp::confess("can't use valueless jbool() test as JSON data")
+      unless defined $_[0]->{JType_value};
+    return $_[0]->{JType_value} ? \1 : \0;
+  }
+  sub _new {
+    my $class = shift;
+    my $test = Test::Deep::all(
+      $BOOL,
+      (@_ ?  Test::Deep::bool($_[0]) : ()),
+    );
+    $test->{JType_value} = $_[0];
+    bless $test, $class;
+    return $test;
+  }
+}
+
 1;
 
 __END__
@@ -136,7 +188,7 @@ Test::Deep::JType - Test::Deep helpers for JSON::Typist data
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 OVERVIEW
 
@@ -208,6 +260,12 @@ makes sure the value was a JSON string, without comparing it to anything.
 C<jtrue> and C<jfalse> are shorthand for C<jbool(1)> and C<jbool(0)>,
 respectively.
 
+As long as they've got a specific value to test for (that is, you called
+C<jstr("foo")> and not C<jstr()>, the tests produced by these routines will
+serialize via a C<convert_blessed>-enabled JSON encode into the appropriate
+types.  This makes it convenient to use these routines for building JSON as
+well as testing it.
+
 =head1 AUTHOR
 
 Ricardo Signes <rjbs@cpan.org>
